@@ -3,12 +3,20 @@
 import urllib.parse
 import logging
 import os
+import re
+import math
 import asyncio
 from .AsyncClient import Client
 from .links import links
-from .html_default_templates import ORIGINAL_HTML, PARAM_BIO, HTML_LIST, BLOCKQUOTE
+from .html_default_templates import (
+    ORIGINAL_HTML,
+    PARAM_BIO,
+    HTML_LIST,
+    BLOCKQUOTE,
+    TEMPLATE,
+)
 from typing import Callable as Function, Any, Dict, Tuple, get_type_hints, List
-from flask import Flask, render_template, request
+from flask import Flask, render_template_string, request
 from waitress import serve
 from threading import Thread
 from time import sleep
@@ -17,8 +25,8 @@ from .exceptions import NamesMustBeAlphanumeric, MustBeRunOnReplitForButtons
 from .utils._uuid import random_characters
 from .post_ql import post
 from .colors import green, blue, purple, red, end, bold_green, bold_blue
+import uuid
 
-app = Flask(__name__)
 _started_buttons = {}
 # line_sep = "-" * os.get_terminal_size().columns
 line_sep = "-" * 80
@@ -33,31 +41,6 @@ logging.basicConfig(
 )
 
 
-@app.route("/<command>/<user>/<choice>/<rand_chars>")
-def _parse_button_commands(command, user, choice, rand_chars):
-    global _started_buttons
-    if request.headers["X-Replit-User-Name"] == "":
-        return render_template(
-            "index.html",
-            html='<center><div><script authed="location.reload()" src="https://auth.util.repl.co/script.js"></script></div></center>',
-        )
-    if (
-        command in _started_buttons
-        and user in _started_buttons[command]
-        and rand_chars in _started_buttons[command][user]
-        and user == request.headers["X-Replit-User-Name"]
-    ):
-        _started_buttons[command][user][rand_chars] = choice
-        return render_template(
-            "index.html",
-            html="<center><h1>your request has been processed, you can close this tab</h1></center>",
-        )
-    else:
-        return render_template(
-            "index.html", html="<center><h1>you cannot do this right now</h1></center>"
-        )
-
-
 class Button:
     def __init__(self, user: str, command: str):
         self.choice = None
@@ -100,8 +83,6 @@ class Bot(Client):
         token: str = None,
         prefix: str = "/",
         bio: str = "",
-        allow_api: bool = False,
-        api_path: str = "/api",
     ) -> None:
         if token is not None:
             super(Client, self).__init__()
@@ -115,9 +96,7 @@ class Bot(Client):
                 if links.docs is not None:
                     await ctx.reply(f"See the docs there {links.docs}")
                 else:
-                    await ctx.reply(
-                        "This bot has not docs. You can check specific commands however"
-                    )
+                    await ctx.reply(self.generate_doc_html())
             else:
                 await ctx.reply(
                     f"description of command: {self.commands[command]['desc']}"
@@ -128,7 +107,7 @@ class Bot(Client):
             __current += (
                 f" You can see the bot docs here {links.docs}"
                 if links.docs is not None
-                else ""
+                else self.generate_doc_html()
             )
             await ctx.reply(__current)
 
@@ -137,11 +116,11 @@ class Bot(Client):
             __current += (
                 f" You can check the bot docs here {links.docs}"
                 if links.docs is not None
-                else ""
+                else self.generate_doc_html()
             )
             await ctx.reply(__current)
 
-        async def _call_when_followed(ctx, person) -> None:
+        async def __default_call_when_followed(ctx, person) -> None:
             logger.info(
                 f"{green}[FILE] BOT.py{end}\n{green}[INFO]{end} @{person.username} followed bot"
             )
@@ -155,6 +134,8 @@ class Bot(Client):
                 "params": {"command": Param(required=False, default="None")},
             }
         }
+        self.doc_html = None  # deadlock for if html generated || stores html
+        self._call_when_followed = __default_call_when_followed
         self._default = __current_default
         self._not_all_required_params_specified = __not_included_params
         self.token = token
@@ -163,14 +144,6 @@ class Bot(Client):
         self.alias = {}
         self.listeners = {}
         self.threads_ = []
-        self._allow_api = allow_api
-        if self._allow_api:
-
-            @app.route(api_path, methods=["POST"])
-            async def _raw_api():
-                kwargs = {"vars": {}, "raw": True}
-                kwargs.update(request.json)
-                return await self.gql(kwargs["query"], kwargs["vars"], kwargs["raw"])
 
     def command(
         self, name: str, thread: bool = False, desc: str = None, alias: List[str] = []
@@ -214,13 +187,13 @@ class Bot(Client):
 
         return wrapper
 
-    async def follower(self, func):
+    def follower(self, func):
         self._call_when_followed = func
 
-    async def default(self, func):
+    def default(self, func):
         self._default = func
 
-    async def fallback_param_not_included_case(self, func):
+    def fallback_param_not_included_case(self, func):
         self._not_all_required_params_specified = func
 
     async def parse_command(self, command: str):
@@ -261,11 +234,11 @@ class Bot(Client):
             output["options"][_current_command] = current.rstrip()
         return output
 
-    async def valid_command(self, resp: Dict[str, Any]) -> Tuple[bool, Dict[str, Any]]:
+    async def valid_command(self, resp: str) -> Tuple[bool, Dict[str, Any]]:
         """validates command. Returns true if is valid `(True, parsed_json)` or false if not `(False, {'None': None})`"""
-        if resp == {} or resp["comment"] == None:
+        if resp is None:
             return (False, {"None": None})
-        parsed = self.parse_command(resp["comment"]["body"])
+        parsed = await self.parse_command(resp)
         if (
             parsed == {}
             or parsed["ping statement"].strip("@") != self.user.username.lower()
@@ -290,8 +263,11 @@ class Bot(Client):
                 output[i] = params[i].type_cast(output[i])
         return (True, output)
 
-    def create_docs(self) -> None:
+    def generate_doc_html(self) -> None:
         """automatically creates the documentation for the bot"""
+        if self.doc_html is not None:
+            return self.doc_html
+
         html = ORIGINAL_HTML.format(
             self.user.username,
             self.prefix,
@@ -344,12 +320,16 @@ class Bot(Client):
                     _param.type_cast.__name__,
                 )
 
-        @app.route("/")
-        def _() -> None:
-            return render_template("index.html", html=html)
+        return html
 
-    def run(self, auto_create_docs: bool = True, token: str = None) -> None:
-        """mainest runner code"""
+    def run(
+        self,
+        token: str = None,
+        auto_create_docs: bool = False,
+        flask_app: Flask = None,
+        daemon: bool = True,
+    ) -> None:
+        """main runner code, input a flask app and auto_create_docs if you want auto created docs"""
         if token is not None and not self.init_:
             super()
             super().__init__(token)
@@ -361,8 +341,51 @@ class Bot(Client):
                 f'{green}[FILE] BOT.py{end}\n{bold_green}[STARTING BOT]{end} Botting "{bold_blue}{client.user.username}{end}"'
             )
 
-        if auto_create_docs:
-            self.create_docs()
+        self.emit("ready", self)
+        self.doc_html = self.generate_doc_html()
+
+        if auto_create_docs and flask_app is None:
+            flask_app = Flask("replit_bot")
+
+        if flask_app is not None:
+
+            if auto_create_docs:
+
+                @flask_app.route("/")
+                def _():
+                    return render_template_string(TEMPLATE, html=self.doc_html)
+
+            else:
+                links.docs = None
+
+            @flask_app.route("/<command>/<user>/<choice>/<rand_chars>")
+            def _parse_button_commands(command, user, choice, rand_chars):
+                global _started_buttons
+                if request.headers["X-Replit-User-Name"] == "":
+                    return render_template_string(
+                        TEMPLATE,
+                        html='<center><div><script authed="location.reload()" src="https://auth.util.repl.co/script.js"></script></div></center>',
+                    )
+                if (
+                    command in _started_buttons
+                    and user in _started_buttons[command]
+                    and rand_chars in _started_buttons[command][user]
+                    and _started_buttons[command][user][rand_chars] is None
+                    and user == request.headers["X-Replit-User-Name"]
+                ):
+                    _started_buttons[command][user][rand_chars] = choice
+                    return render_template_string(
+                        TEMPLATE,
+                        html="<center><h1>your request has been processed, you can close this tab</h1></center>",
+                    )
+                else:
+                    return render_template_string(
+                        TEMPLATE,
+                        html="<center><h1>you cannot do this right now</h1></center>",
+                    )
+
+        else:
+            links.docs = None
 
         @self.on("notification")
         async def _run(notif_id, notif) -> None:
@@ -385,10 +408,17 @@ class Bot(Client):
                 for i in dir(self):
                     if not i.startswith("__") and not i.endswith("__"):
                         setattr(notif.comment, i, getattr(self, i))
-                parsed_json = await self.parse_command(notif.comment.body)
-                if "command" in parsed_json and (
-                    parsed_json["command"] in self.commands
-                    or parsed_json["command"] in self.alias
+                valid_first, parsed_json = await self.valid_command(notif.comment.body)
+                if not valid_first:
+                    return
+
+                if (
+                    "command" in parsed_json
+                    and (
+                        parsed_json["command"] in self.commands
+                        or parsed_json["command"] in self.alias
+                    )
+                    and valid_first
                 ):
                     c = parsed_json["command"]
                     if parsed_json["command"] in self.alias:
@@ -396,7 +426,7 @@ class Bot(Client):
                     valid, kwargs = await self.get_kwargs(
                         self.commands[c], parsed_json["options"]
                     )
-                    if valid:
+                    if valid and valid_first:
                         notif.comment.button = Button(notif.comment.author.username, c)
 
                         logger.info(
@@ -413,16 +443,18 @@ class Bot(Client):
                             await self.commands[c]["call"](notif.comment, **kwargs)
                         else:
                             await self.commands[c]["call"](notif.comment, **kwargs)
-                    else:
+                    elif valid_first:
                         await self._not_all_required_params_specified(notif.comment)
-                elif "command" in parsed_json and (
-                    parsed_json["command"] in self.listeners
+                elif (
+                    "command" in parsed_json
+                    and (parsed_json["command"] in self.listeners)
+                    and valid_first
                 ):
                     c = parsed_json["command"]
                     valid, kwargs = await self.get_kwargs(
                         self.listeners[c], parsed_json["options"]
                     )
-                    if valid:
+                    if valid and valid_first:
                         notif.comment.button = Button(notif.comment.user.username, c)
                         logger.info(
                             f"{green}[FILE] BOT.py{end}\n{green}[INFO] logging command{end}.\n\t{blue}[SUMMARY]{end} {green}command successful{end}\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
@@ -439,22 +471,166 @@ class Bot(Client):
                                 await command[c]["call"](notif.comment, **kwargs)
                             else:
                                 await command[c]["call"](notif.comment, **kwargs)
-                    else:
+                    elif valid_first:
                         logger.info(
                             f"{green}[FILE] BOT.py{end}\n{green}[INFO]{end} logging command\n\t{blue}[SUMMARY]{end} unsucessful: {red}Not all required params specified{end}.\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
                         )
                         await self._not_all_required_params_specified(notif.comment)
-                elif "command" in parsed_json:
+                elif "command" in parsed_json and valid_first:
                     logger.info(
                         f"{green}[FILE] BOT.py{end}\n{green}[INFO]{end} logging command\n\t{blue}[SUMMARY]{end} unsuccessful: {red}Invalid command{end}.\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
                     )
                     await self._default(notif.comment)
+            elif __typename == "AnnotationNotification":
+                super_self = self
 
-        if auto_create_docs:
+                class CurrentThread:
+                    def __init__(self, data) -> None:
+                        self.id = data["id"]
+                        self.anchor_id = data["anchor"]["id"]
+                        self.user = None
+
+                        async def __temp_wrapper():
+                            self.user = await super_self.users.fetch(
+                                data["user"]["username"]
+                            )
+
+                        loop = asyncio.get_event_loop()
+                        loop.create_task(__temp_wrapper())
+                        self.body = data["content"]["text"]
+
+                res = await self.gql(
+                    "getReplAnnotations",
+                    {"url": notif.url},
+                )
+                repl_annotations = res["repl"]
+                if repl_annotations is None:
+                    return
+                anchor = list(
+                    filter(
+                        lambda x: x["isGeneral"], repl_annotations["annotationAnchors"]
+                    )
+                )[0]
+                if not anchor:
+                    return
+                messages = list(map(lambda x: CurrentThread(x), anchor["messages"]))
+                message = list(
+                    filter(
+                        lambda x: x.body.startswith("@" + self.user.username), messages
+                    )
+                )[0]
+                await self.gql(
+                    "markMessageAsSeen",
+                    {"replId": repl_annotations["id"], "anchorId": anchor["id"]},
+                )
+
+                class CurrentCtx:
+                    def __init__(self):
+                        self.repl_id = repl_annotations["id"]
+                        self.anchor_id = anchor["id"]
+
+                    async def reply(self, body) -> None:
+                        return await self.gql(
+                            "createChatMessage",
+                            vars={
+                                "replId": self.repl_id,
+                                "anchorId": self.anchor_id,
+                                "annotationMessage": {
+                                    "id": str(uuid.uuid4()),
+                                    "text": body,
+                                },
+                            },
+                        )
+
+                ctx = CurrentCtx()
+                for i in dir(self):
+                    if not i.startswith("__") and not i.endswith("__"):
+                        setattr(ctx, i, getattr(self, i))
+                for i in dir(message):
+                    if not i.startswith("__") and not i.endswith("__"):
+                        setattr(ctx, i, getattr(message, i))
+
+                ctx.author = ctx.user
+                ctx.author.mention = "@" + ctx.user.username
+                valid_first, parsed_json = await self.valid_command(ctx.body)
+                if not valid_first:
+                    return
+
+                if (
+                    "command" in parsed_json
+                    and (
+                        parsed_json["command"] in self.commands
+                        or parsed_json["command"] in self.alias
+                    )
+                    and valid_first
+                ):
+                    c = parsed_json["command"]
+                    if parsed_json["command"] in self.alias:
+                        c = self.alias[c]
+                    valid, kwargs = await self.get_kwargs(
+                        self.commands[c], parsed_json["options"]
+                    )
+                    if valid and valid_first:
+                        ctx.button = Button(ctx.author.username, c)
+
+                        logger.info(
+                            f"{green}[FILE] BOT.py{end}\n{green}[INFO] logging command{end}.\n\t{blue}[SUMMARY]{end} {green}command successful{end}\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
+                        )
+                        if self.commands[c]["thread"]:
+                            # t = Thread(
+                            #     target=self.commands[c]["call"],
+                            #     args=(notif.comment,),
+                            #     kwargs=kwargs,
+                            # )
+                            # t.start()
+                            # self.threads_.append(t)
+                            await self.commands[c]["call"](ctx, **kwargs)
+                        else:
+                            await self.commands[c]["call"](ctx, **kwargs)
+                    elif valid_first:
+                        await self._not_all_required_params_specified(ctx)
+                elif (
+                    "command" in parsed_json
+                    and (parsed_json["command"] in self.listeners)
+                    and valid_first
+                ):
+                    c = parsed_json["command"]
+                    valid, kwargs = await self.get_kwargs(
+                        self.listeners[c], parsed_json["options"]
+                    )
+                    if valid and valid_first:
+                        ctx.button = Button(ctx.user.username, c)
+                        logger.info(
+                            f"{green}[FILE] BOT.py{end}\n{green}[INFO] logging command{end}.\n\t{blue}[SUMMARY]{end} {green}command successful{end}\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
+                        )
+                        for command in self.listeners[c]:
+                            if command[c]["thread"]:
+                                # t = Thread(
+                                #     target=command[c]["call"],
+                                #     args=(notif.comment,),
+                                #     kwargs=kwargs,
+                                # )
+                                # t.start()
+                                # self.threads_.append(t)
+                                await command[c]["call"](ctx, **kwargs)
+                            else:
+                                await command[c]["call"](ctx, **kwargs)
+                    elif valid_first:
+                        logger.info(
+                            f"{green}[FILE] BOT.py{end}\n{green}[INFO]{end} logging command\n\t{blue}[SUMMARY]{end} unsucessful: {red}Not all required params specified{end}.\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
+                        )
+                        await self._not_all_required_params_specified(ctx)
+                elif "command" in parsed_json and valid_first:
+                    logger.info(
+                        f"{green}[FILE] BOT.py{end}\n{green}[INFO]{end} logging command\n\t{blue}[SUMMARY]{end} unsuccessful: {red}Invalid command{end}.\n\t{purple}[EXTRA]{end} Requested command: {parsed_json['command']}"
+                    )
+                    await self._default(ctx)
+
+        if flask_app is not None:
             Thread(
                 target=serve,
-                kwargs={"app": app, "host": "0.0.0.0", "port": 8080},
-                daemon=True,
+                kwargs={"app": flask_app, "host": "0.0.0.0", "port": 8080},
+                daemon=daemon,
             ).start()
         loop = asyncio.get_event_loop()
         loop.run_until_complete(self.user.notifications.startEvents())
